
module test_top
(
	input            I_clk     , //50Mhz
	input            I_rst_n   , //key1
    inout            IO_sda    ,
    input            I_scl     ,
	output  		 O_tx      ,
	input   		 I_rx      ,
	output     [3:0] O_led     
);

//===================================================
reg  [31:0] run_cnt;
wire        running;

//apb reg
wire [31:0] paddr;
wire        penable;
wire [31:0] prdata;
wire        pready;
wire        psel;
wire [31:0] pwdata;
wire        pwrite;

wire        ma_we;

//===================================================
//LED灯测试
always @(posedge I_clk or negedge I_rst_n)
begin
	if(!I_rst_n)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd50_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;

assign  O_led[0] = running; 
assign  O_led[1] = 1'b0;
assign  O_led[2] = 1'b0;
assign  O_led[3] = 1'b0;

//===========================================================================
EDID_PROM_Top  EDID_PROM_Top_inst 
( 
	.I_clk      (I_clk      ),//>= 5MHz, <=200MHz 
	.I_rst_n    (I_rst_n    ), 
    .I_mema_clk (I_clk      ),
    .I_mema_we  (ma_we      ),
    .I_mema_addr(paddr[7:0] ),  
    .I_mema_di  (pwdata[7:0]),
	.I_scl      (I_scl      ),
	.IO_sda     (IO_sda     )
);

assign ma_we = pwrite & psel & penable;
assign prdata = 32'd0;

//========================================================
//UART to APB bus
uart_apb_mst_top #
(	
	.BAUDRATE        (115200    ),//buad rate
	.FREQ            (50_000_000),//system clock, MHz
    .FIFO_DEPTH_WIDTH(8         ) //Depth=2^FIFO_DEPTH_WIDTH
)
uart_apb_mst_top_inst
(
	.rst_n             (I_rst_n   ),
	.clk_sample        (I_clk     ),
	.rx                (I_rx      ),
	.tx                (O_tx      ),
    .paddr             (paddr     ),//apb port 
    .penable           (penable   ),
    .prdata            (prdata    ),
    .pready            (1'b1      ),
    .psel              (psel      ),
    .pwdata            (pwdata    ),
    .pwrite            (pwrite    )
); 


endmodule