//---------------------------------------------------------------------
// File name         : cmd_gen.v
// Module name       : cmd_gen
// Module Description: Generate gmm command for simulation
// Created by        : caojie
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date      | AUTHOR      |    DESCRIPTION
// ---------------------------------------------------------------------
//   1.0   |  8-Jul-2023| Caojie      |      initial
// ---------------------------------------------------------------------

`timescale 1ns / 1ps


`define CHAR_SPACE      8'h20
`define CHAR_R_UP       8'h52
`define CHAR_r_LO       8'h72
`define CHAR_W_UP       8'h57
`define CHAR_w_LO       8'h77
`define CHAR_0          8'h30
`define CHAR_1          8'h31
`define CHAR_2          8'h32
`define CHAR_3          8'h33
`define CHAR_4          8'h34
`define CHAR_5          8'h35
`define CHAR_6          8'h36
`define CHAR_7          8'h37
`define CHAR_8          8'h38
`define CHAR_9          8'h39
`define CHAR_A_UP       8'h41
`define CHAR_B_UP       8'h42
`define CHAR_C_UP       8'h43
`define CHAR_D_UP       8'h44
`define CHAR_E_UP       8'h45
`define CHAR_F_UP       8'h46
`define CHAR_a_LO       8'h61
`define CHAR_b_LO       8'h62
`define CHAR_c_LO       8'h63
`define CHAR_d_LO       8'h64
`define CHAR_e_LO       8'h65
`define CHAR_f_LO       8'h66  


module cmd_gen
(
    input             clk        , 
    input             rst_n      , 
    output reg        cmd_finish ,
    output reg [31:0] paddr        ,//APB address
    output reg        penable      ,
    input      [31:0] prdata       ,
    input             pready       ,
    output reg        psel         ,
    output reg [31:0] pwdata       ,
    output reg        pwrite   
);

//============================================
reg  [2:0]  cnt;
reg  [31:0] icnt;

//============================================================
localparam DW = 152;//instruction character string, 1 character need 8bit, 19*8 = 152
localparam N  = 256;//instruction number, must match with actual memory depth

//---------------------------------------
//Adding UART instructions here
wire  [DW-1:0] memc   [0:N-1] ; //

assign memc[0  ] = "w 00000000 00000000";
assign memc[1  ] = "w 00000001 000000ff";
assign memc[2  ] = "w 00000002 000000ff";
assign memc[3  ] = "w 00000003 000000ff";
assign memc[4  ] = "w 00000004 000000ff";
assign memc[5  ] = "w 00000005 000000ff";
assign memc[6  ] = "w 00000006 000000ff";
assign memc[7  ] = "w 00000007 00000000";
assign memc[8  ] = "w 00000008 00000068";
assign memc[9  ] = "w 00000009 0000006c";
assign memc[10 ] = "w 0000000a 00000000";
assign memc[11 ] = "w 0000000b 00000000";
assign memc[12 ] = "w 0000000c 00000001";
assign memc[13 ] = "w 0000000d 00000001";
assign memc[14 ] = "w 0000000e 00000001";
assign memc[15 ] = "w 0000000f 00000001";
assign memc[16 ] = "w 00000010 00000001";
assign memc[17 ] = "w 00000011 0000001d";
assign memc[18 ] = "w 00000012 00000001";
assign memc[19 ] = "w 00000013 00000004";
assign memc[20 ] = "w 00000014 00000080";
assign memc[21 ] = "w 00000015 0000001c";
assign memc[22 ] = "w 00000016 00000003";
assign memc[23 ] = "w 00000017 00000078";
assign memc[24 ] = "w 00000018 0000000a";
assign memc[25 ] = "w 00000019 00000005";
assign memc[26 ] = "w 0000001a 00000048";
assign memc[27 ] = "w 0000001b 000000a1";
assign memc[28 ] = "w 0000001c 0000005a";
assign memc[29 ] = "w 0000001d 00000052";
assign memc[30 ] = "w 0000001e 000000a5";
assign memc[31 ] = "w 0000001f 00000024";
assign memc[32 ] = "w 00000020 00000017";
assign memc[33 ] = "w 00000021 0000004d";
assign memc[34 ] = "w 00000022 0000005a";
assign memc[35 ] = "w 00000023 00000000";
assign memc[36 ] = "w 00000024 00000000";
assign memc[37 ] = "w 00000025 00000000";
assign memc[38 ] = "w 00000026 00000001";
assign memc[39 ] = "w 00000027 00000000";
assign memc[40 ] = "w 00000028 00000001";
assign memc[41 ] = "w 00000029 00000001";
assign memc[42 ] = "w 0000002a 00000001";
assign memc[43 ] = "w 0000002b 00000001";
assign memc[44 ] = "w 0000002c 00000001";
assign memc[45 ] = "w 0000002d 00000001";
assign memc[46 ] = "w 0000002e 00000001";
assign memc[47 ] = "w 0000002f 00000001";
assign memc[48 ] = "w 00000030 00000001";
assign memc[49 ] = "w 00000031 00000001";
assign memc[50 ] = "w 00000032 00000001";
assign memc[51 ] = "w 00000033 00000001";
assign memc[52 ] = "w 00000034 00000001";
assign memc[53 ] = "w 00000035 00000001";
assign memc[54 ] = "w 00000036 00000064";
assign memc[55 ] = "w 00000037 00000019";
assign memc[56 ] = "w 00000038 00000000";
assign memc[57 ] = "w 00000039 00000040";
assign memc[58 ] = "w 0000003a 00000041";
assign memc[59 ] = "w 0000003b 00000000";
assign memc[60 ] = "w 0000003c 00000026";
assign memc[61 ] = "w 0000003d 00000030";
assign memc[62 ] = "w 0000003e 00000018";
assign memc[63 ] = "w 0000003f 00000088";
assign memc[64 ] = "w 00000040 00000036";
assign memc[65 ] = "w 00000041 00000000";
assign memc[66 ] = "w 00000042 00000000";
assign memc[67 ] = "w 00000043 00000000";
assign memc[68 ] = "w 00000044 00000043";
assign memc[69 ] = "w 00000045 00000000";
assign memc[70 ] = "w 00000046 00000000";
assign memc[71 ] = "w 00000047 00000018";
assign memc[72 ] = "w 00000048 00000000";
assign memc[73 ] = "w 00000049 00000000";
assign memc[74 ] = "w 0000004a 00000000";
assign memc[75 ] = "w 0000004b 00000010";
assign memc[76 ] = "w 0000004c 00000000";
assign memc[77 ] = "w 0000004d 00000000";
assign memc[78 ] = "w 0000004e 00000000";
assign memc[79 ] = "w 0000004f 00000000";
assign memc[80 ] = "w 00000050 00000000";
assign memc[81 ] = "w 00000051 00000000";
assign memc[82 ] = "w 00000052 00000000";
assign memc[83 ] = "w 00000053 00000000";
assign memc[84 ] = "w 00000054 00000000";
assign memc[85 ] = "w 00000055 00000000";
assign memc[86 ] = "w 00000056 00000000";
assign memc[87 ] = "w 00000057 00000000";
assign memc[88 ] = "w 00000058 00000000";
assign memc[89 ] = "w 00000059 00000000";
assign memc[90 ] = "w 0000005a 00000000";
assign memc[91 ] = "w 0000005b 00000000";
assign memc[92 ] = "w 0000005c 00000000";
assign memc[93 ] = "w 0000005d 00000010";
assign memc[94 ] = "w 0000005e 00000000";
assign memc[95 ] = "w 0000005f 00000000";
assign memc[96 ] = "w 00000060 00000000";
assign memc[97 ] = "w 00000061 00000000";
assign memc[98 ] = "w 00000062 00000000";
assign memc[99 ] = "w 00000063 00000000";
assign memc[100] = "w 00000064 00000000";
assign memc[101] = "w 00000065 00000000";
assign memc[102] = "w 00000066 00000000";
assign memc[103] = "w 00000067 00000000";
assign memc[104] = "w 00000068 00000000";
assign memc[105] = "w 00000069 00000000";
assign memc[106] = "w 0000006a 00000000";
assign memc[107] = "w 0000006b 00000000";
assign memc[108] = "w 0000006c 00000000";
assign memc[109] = "w 0000006d 00000000";
assign memc[110] = "w 0000006e 00000000";
assign memc[111] = "w 0000006f 00000010";
assign memc[112] = "w 00000070 00000000";
assign memc[113] = "w 00000071 00000000";
assign memc[114] = "w 00000072 00000000";
assign memc[115] = "w 00000073 00000000";
assign memc[116] = "w 00000074 00000000";
assign memc[117] = "w 00000075 00000000";
assign memc[118] = "w 00000076 00000000";
assign memc[119] = "w 00000077 00000000";
assign memc[120] = "w 00000078 00000000";
assign memc[121] = "w 00000079 00000000";
assign memc[122] = "w 0000007a 00000000";
assign memc[123] = "w 0000007b 00000000";
assign memc[124] = "w 0000007c 00000000";
assign memc[125] = "w 0000007d 00000000";
assign memc[126] = "w 0000007e 00000001";
assign memc[127] = "w 0000007f 00000004";
assign memc[128] = "w 00000080 00000000";
assign memc[129] = "w 00000081 00000003";
assign memc[130] = "w 00000082 0000001d";
assign memc[131] = "w 00000083 000000f2";
assign memc[132] = "w 00000084 0000004a";
assign memc[133] = "w 00000085 00000005";
assign memc[134] = "w 00000086 00000084";
assign memc[135] = "w 00000087 00000003";
assign memc[136] = "w 00000088 00000002";
assign memc[137] = "w 00000089 00000001";
assign memc[138] = "w 0000008a 00000012";
assign memc[139] = "w 0000008b 00000093";
assign memc[140] = "w 0000008c 00000014";
assign memc[141] = "w 0000008d 00000016";
assign memc[142] = "w 0000008e 00000007";
assign memc[143] = "w 0000008f 00000023";
assign memc[144] = "w 00000090 00000009";
assign memc[145] = "w 00000091 00000007";
assign memc[146] = "w 00000092 00000007";
assign memc[147] = "w 00000093 00000083";
assign memc[148] = "w 00000094 00000001";
assign memc[149] = "w 00000095 00000000";
assign memc[150] = "w 00000096 00000000";
assign memc[151] = "w 00000097 00000065";
assign memc[152] = "w 00000098 00000003";
assign memc[153] = "w 00000099 0000000c";
assign memc[154] = "w 0000009a 00000000";
assign memc[155] = "w 0000009b 00000010";
assign memc[156] = "w 0000009c 00000000";
assign memc[157] = "w 0000009d 0000005f";
assign memc[158] = "w 0000009e 0000000c";
assign memc[159] = "w 0000009f 00000000";
assign memc[160] = "w 000000a0 00000064";
assign memc[161] = "w 000000a1 000000f0";
assign memc[162] = "w 000000a2 00000078";
assign memc[163] = "w 000000a3 0000000e";
assign memc[164] = "w 000000a4 00000000";
assign memc[165] = "w 000000a5 00000034";
assign memc[166] = "w 000000a6 00000014";
assign memc[167] = "w 000000a7 00000082";
assign memc[168] = "w 000000a8 00000000";
assign memc[169] = "w 000000a9 000000db";
assign memc[170] = "w 000000aa 000000e1";
assign memc[171] = "w 000000ab 00000000";
assign memc[172] = "w 000000ac 00000000";
assign memc[173] = "w 000000ad 00000000";
assign memc[174] = "w 000000ae 00000018";
assign memc[175] = "w 000000af 00000055";
assign memc[176] = "w 000000b0 00000004";
assign memc[177] = "w 000000b1 00000000";
assign memc[178] = "w 000000b2 00000064";
assign memc[179] = "w 000000b3 00000050";
assign memc[180] = "w 000000b4 00000078";
assign memc[181] = "w 000000b5 0000000e";
assign memc[182] = "w 000000b6 00000000";
assign memc[183] = "w 000000b7 00000034";
assign memc[184] = "w 000000b8 00000014";
assign memc[185] = "w 000000b9 00000082";
assign memc[186] = "w 000000ba 00000000";
assign memc[187] = "w 000000bb 000000db";
assign memc[188] = "w 000000bc 000000e1";
assign memc[189] = "w 000000bd 00000000";
assign memc[190] = "w 000000be 00000000";
assign memc[191] = "w 000000bf 00000000";
assign memc[192] = "w 000000c0 00000019";
assign memc[193] = "w 000000c1 00000000";
assign memc[194] = "w 000000c2 00000000";
assign memc[195] = "w 000000c3 00000000";
assign memc[196] = "w 000000c4 00000000";
assign memc[197] = "w 000000c5 00000000";
assign memc[198] = "w 000000c6 00000000";
assign memc[199] = "w 000000c7 00000000";
assign memc[200] = "w 000000c8 00000000";
assign memc[201] = "w 000000c9 00000000";
assign memc[202] = "w 000000ca 00000000";
assign memc[203] = "w 000000cb 00000000";
assign memc[204] = "w 000000cc 00000000";
assign memc[205] = "w 000000cd 00000000";
assign memc[206] = "w 000000ce 00000000";
assign memc[207] = "w 000000cf 00000000";
assign memc[208] = "w 000000d0 00000000";
assign memc[209] = "w 000000d1 00000000";
assign memc[210] = "w 000000d2 00000000";
assign memc[211] = "w 000000d3 00000000";
assign memc[212] = "w 000000d4 00000000";
assign memc[213] = "w 000000d5 00000000";
assign memc[214] = "w 000000d6 00000000";
assign memc[215] = "w 000000d7 00000000";
assign memc[216] = "w 000000d8 00000000";
assign memc[217] = "w 000000d9 00000000";
assign memc[218] = "w 000000da 00000000";
assign memc[219] = "w 000000db 00000000";
assign memc[220] = "w 000000dc 00000000";
assign memc[221] = "w 000000dd 00000000";
assign memc[222] = "w 000000de 00000000";
assign memc[223] = "w 000000df 00000000";
assign memc[224] = "w 000000e0 00000000";
assign memc[225] = "w 000000e1 00000000";
assign memc[226] = "w 000000e2 00000000";
assign memc[227] = "w 000000e3 00000000";
assign memc[228] = "w 000000e4 00000000";
assign memc[229] = "w 000000e5 00000000";
assign memc[230] = "w 000000e6 00000000";
assign memc[231] = "w 000000e7 00000000";
assign memc[232] = "w 000000e8 00000000";
assign memc[233] = "w 000000e9 00000000";
assign memc[234] = "w 000000ea 00000000";
assign memc[235] = "w 000000eb 00000000";
assign memc[236] = "w 000000ec 00000000";
assign memc[237] = "w 000000ed 00000000";
assign memc[238] = "w 000000ee 00000000";
assign memc[239] = "w 000000ef 00000000";
assign memc[240] = "w 000000f0 00000000";
assign memc[241] = "w 000000f1 00000000";
assign memc[242] = "w 000000f2 00000000";
assign memc[243] = "w 000000f3 00000000";
assign memc[244] = "w 000000f4 00000000";
assign memc[245] = "w 000000f5 00000000";
assign memc[246] = "w 000000f6 00000000";
assign memc[247] = "w 000000f7 00000000";
assign memc[248] = "w 000000f8 00000000";
assign memc[249] = "w 000000f9 00000000";
assign memc[250] = "w 000000fa 00000000";
assign memc[251] = "w 000000fb 00000000";
assign memc[252] = "w 000000fc 00000000";
assign memc[253] = "w 000000fd 00000000";
assign memc[254] = "w 000000fe 00000000";
assign memc[255] = "w 000000ff 000000e8";

//============================================================
always@(posedge clk or negedge rst_n)
begin
    if (!rst_n) 
		cnt <= 3'd0;
	else 
		cnt <= cnt + 1'b1;
end

always@(posedge clk or negedge rst_n)
begin
    if (!rst_n) 
		icnt <= 32'd0;
	else if(cnt==3'd7)
		icnt <= icnt + 1'b1;
    else
        icnt <= icnt;
end

always@(posedge clk or negedge rst_n)
begin
    if (!rst_n) 
		cmd_finish <= 1'd0;
	else if(icnt>=N) 
		cmd_finish <= 1'd1;
    else
        cmd_finish <= 1'd0;
end

//============================================================
//'R' / 'r'  read   
//'W' / 'w'  write

always@(posedge clk or negedge rst_n)
begin
    if (!rst_n) 
        begin
            paddr   <= 32'd0;
			pwrite  <= 1'b0;
            psel    <= 1'b0;
            penable <= 1'b0;
            pwdata  <= 32'd0;
        end
    else if(cnt==3'd1 && icnt<N)
        begin
            if(memc[icnt][DW-1:DW-8] == "R" || 
               memc[icnt][DW-1:DW-8] == "r")
                begin
                    //read
                    paddr   <= {s2h(memc[icnt][DW-17:DW-24]),
                                s2h(memc[icnt][DW-25:DW-32]),
                                s2h(memc[icnt][DW-33:DW-40]),
                                s2h(memc[icnt][DW-41:DW-48]),
                                s2h(memc[icnt][DW-49:DW-56]),
                                s2h(memc[icnt][DW-57:DW-64]),
                                s2h(memc[icnt][DW-65:DW-72]),
                                s2h(memc[icnt][DW-73:DW-80])};
                    pwrite  <= 1'd0 ;
                    psel    <= 1'b1;
                    penable <= 1'b0;
                    pwdata  <= 32'd0;	
                end
            else if(memc[icnt][DW-1:DW-8] == "W" || 
                    memc[icnt][DW-1:DW-8] == "w")
                begin
                    //write
                    paddr   <= {s2h(memc[icnt][DW-17:DW-24]),
                                s2h(memc[icnt][DW-25:DW-32]),
                                s2h(memc[icnt][DW-33:DW-40]),
                                s2h(memc[icnt][DW-41:DW-48]),
                                s2h(memc[icnt][DW-49:DW-56]),
                                s2h(memc[icnt][DW-57:DW-64]),
                                s2h(memc[icnt][DW-65:DW-72]),
                                s2h(memc[icnt][DW-73:DW-80])};
                    pwrite  <= 1'd1 ;
                    psel    <= 1'b1;
                    penable <= 1'b0;
                    pwdata  <= 32'd0;	
                end
            else
                begin
                    $display("Please enter right instruction format:");
                    $display("Read command format : R xxxx     ");
                    $display("Write command format: W xxxx xxxx");
                    $stop;
                end
        end
    else if(cnt==3'd2 && icnt<N)
        begin
            if(memc[icnt][DW-1:DW-8] == "R" || 
               memc[icnt][DW-1:DW-8] == "r")
                begin
                    //read
                    paddr   <= {s2h(memc[icnt][DW-17:DW-24]),
                                s2h(memc[icnt][DW-25:DW-32]),
                                s2h(memc[icnt][DW-33:DW-40]),
                                s2h(memc[icnt][DW-41:DW-48]),
                                s2h(memc[icnt][DW-49:DW-56]),
                                s2h(memc[icnt][DW-57:DW-64]),
                                s2h(memc[icnt][DW-65:DW-72]),
                                s2h(memc[icnt][DW-73:DW-80])};
                    pwrite  <= 1'd0 ;
                    psel    <= 1'b1;
                    penable <= 1'b1;
                    pwdata  <= 32'd0;	
                end
            else if(memc[icnt][DW-1:DW-8] == "W" || 
                    memc[icnt][DW-1:DW-8] == "w")
                begin
                    //write
                    paddr   <= {s2h(memc[icnt][DW-17:DW-24]),
                                s2h(memc[icnt][DW-25:DW-32]),
                                s2h(memc[icnt][DW-33:DW-40]),
                                s2h(memc[icnt][DW-41:DW-48]),
                                s2h(memc[icnt][DW-49:DW-56]),
                                s2h(memc[icnt][DW-57:DW-64]),
                                s2h(memc[icnt][DW-65:DW-72]),
                                s2h(memc[icnt][DW-73:DW-80])};
                    pwrite  <= 1'd1 ;
                    psel    <= 1'b1;
                    penable <= 1'b1;
                    pwdata  <= {s2h(memc[icnt][DW-89:DW-96]),
                                s2h(memc[icnt][DW-97:DW-104]),
                                s2h(memc[icnt][DW-105:DW-112]),
                                s2h(memc[icnt][DW-113:DW-120]),
                                s2h(memc[icnt][DW-121:DW-128]),
                                s2h(memc[icnt][DW-129:DW-136]),
                                s2h(memc[icnt][DW-137:DW-144]),
                                s2h(memc[icnt][DW-145:DW-152])};	
                end
            else
                begin
                    $display("Please enter right instruction format:");
                    $display("Read command format : R xxxx     ");
                    $display("Write command format: W xxxx xxxx");
                    $stop;
                end
        end
    else
        begin
            paddr   <= 32'd0;
			pwrite  <= 1'b0;
            psel    <= 1'b0;
            penable <= 1'b0;
            pwdata  <= 32'd0;
        end
end

function [3:0] s2h;
    input [7:0] str;
    begin
        s2h = (str==`CHAR_0)    ? 4'h0 :
              (str==`CHAR_1)    ? 4'h1 :
              (str==`CHAR_2)    ? 4'h2 :
              (str==`CHAR_3)    ? 4'h3 :
              (str==`CHAR_4)    ? 4'h4 :
              (str==`CHAR_5)    ? 4'h5 :
              (str==`CHAR_6)    ? 4'h6 :
              (str==`CHAR_7)    ? 4'h7 :
              (str==`CHAR_8)    ? 4'h8 :
              (str==`CHAR_9)    ? 4'h9 :
              (str==`CHAR_A_UP) ? 4'ha :
              (str==`CHAR_B_UP) ? 4'hb :
              (str==`CHAR_C_UP) ? 4'hc :
              (str==`CHAR_D_UP) ? 4'hd :
              (str==`CHAR_E_UP) ? 4'he :
              (str==`CHAR_F_UP) ? 4'hf :
              (str==`CHAR_a_LO) ? 4'ha :
              (str==`CHAR_b_LO) ? 4'hb :
              (str==`CHAR_c_LO) ? 4'hc :
              (str==`CHAR_d_LO) ? 4'hd :
              (str==`CHAR_e_LO) ? 4'he :
              (str==`CHAR_f_LO) ? 4'hf : 4'hf;
    end
endfunction

endmodule


                  