
`timescale 1ns/1ps

module master_sram_iic 
(
     I_CLK,   
     I_RESETN,
     start,
     O_TX_EN, 
     O_WADDR, 
     O_WDATA, 
     O_RX_EN,
     O_RADDR, 
     I_RDATA,     
     cstate_flag,
     error_flag
);

 input           I_CLK;
 input           I_RESETN;
 input           start; 
 output          O_TX_EN;
 output [2:0]    O_WADDR;
 output [8-1:0]  O_WDATA;   
 output          O_RX_EN;  
 output [2:0]    O_RADDR;
 input  [8-1:0]  I_RDATA;
 output          error_flag;
 output          cstate_flag; 
 
//////////////////////////////////////////////////////////////////////////
//    Internal Wires/Registers
 reg                          O_TX_EN;
 reg [2:0]                    O_WADDR;
 reg [8-1:0]     O_WDATA;
 reg                          O_RX_EN;
 reg [2:0]                    O_RADDR;  

//define reg address of IIC-Master
 parameter     PRERLO_ADDR      = 3'b000;
 parameter     PRERHI_ADDR      = 3'b001;
 parameter     CTR_ADDR         = 3'b010;
 parameter     TXR_ADDR         = 3'b011; 
 parameter     RXR_ADDR         = 3'b011; 
 parameter     CR_ADDR          = 3'b100; 
 parameter     SR_ADDR          = 3'b100;

 parameter  TX_DATA     = 8'b00000110;  //1e: 30
 

 parameter    CLK_Div_L    = 8'h63;  //
 parameter    CLK_Div_H    = 8'h00;  //

 parameter    EN_IP        = 8'h80; 
                           
 parameter     STA_WR_CR    = 8'h90;    //start+write_ack
 parameter     WR_CR        = 8'h10; //write+ack
 parameter    STP_WR_CR    = 8'h50; //stop+write+ack
 parameter    STP_CR        = 8'h40; //stop+write+ack
                          
 parameter     RD_CR        = 8'h20;//read+ack
 parameter     STP_RD_NCR    = 8'h68;//stop+read+nack
 
 parameter     RD          = 1'b1;
 parameter     WR             = 1'b0;
 
//define the address of IIC-Slave device 
 parameter     DEV_ADDR    = 7'b1010_000;
 
//状态机
parameter IDLE             = 8'd0;
parameter PERSCALE0_REG    = 8'd1;
parameter PERSCALE1_REG    = 8'd2;
parameter MASTER_EN_REG    = 8'd3;

parameter WR_DEVICE_ADDRW  = 8'd4;
parameter WR_CMD90         = 8'd5;
parameter WR_RDTIP01       = 8'd6;
parameter WR_REG_ADDR      = 8'd7; //addr
parameter WR_CMD10_1       = 8'd8;
parameter WR_RDTIP02       = 8'd9;
parameter WR_DATA          = 8'd10; //data
parameter WR_CMD50_2       = 8'd11;
parameter WR_RDTIP03       = 8'd12;

parameter RD_DEVICE_ADDRW  = 8'd13;
parameter RD_CMD90_1       = 8'd14;
parameter RD_RDTIP01       = 8'd15;
parameter RD_REG_ADDR      = 8'd16; //addr
parameter RD_CMD10_1       = 8'd17;
parameter RD_RDTIP02       = 8'd18;
parameter RD_DEVICE_ADDRR  = 8'd19;
parameter RD_CMD90_2       = 8'd20;
parameter RD_RDTIP03       = 8'd21;
parameter RD_CMD20         = 8'd22;
parameter RD_RDTIP04       = 8'd23;
parameter RD_DATA          = 8'd24; //data
parameter RD_CMD68         = 8'd25;
parameter RD_RDTIP05       = 8'd26;

reg [7:0] state;
 
 reg[1:0]    wr_reg;
 reg[1:0]    rd_reg;

 reg[1:0]    wr_rd_stop;
 
 
 reg[7:0]    sr_data0;
 
 reg        error_reg = 0;
 reg        cstate_flag_reg = 0;
 
 reg        start_dl;
 
 reg[2:0]   rdrpt_cnt;

//-------------------------------------------------------------------
 localparam NUM = 255;
 wire [7:0] addr_mem [0:NUM]; 
 
 reg  [31:0] mem_cnt;

//-------------------------------------------------------------------
assign addr_mem[0  ] = 8'd0  ;
assign addr_mem[1  ] = 8'd1  ;
assign addr_mem[2  ] = 8'd2  ;
assign addr_mem[3  ] = 8'd3  ;
assign addr_mem[4  ] = 8'd4  ;
assign addr_mem[5  ] = 8'd5  ;
assign addr_mem[6  ] = 8'd6  ;
assign addr_mem[7  ] = 8'd7  ;
assign addr_mem[8  ] = 8'd8  ;
assign addr_mem[9  ] = 8'd9  ;
assign addr_mem[10 ] = 8'd10 ;
assign addr_mem[11 ] = 8'd11 ;
assign addr_mem[12 ] = 8'd12 ;
assign addr_mem[13 ] = 8'd13 ;
assign addr_mem[14 ] = 8'd14 ;
assign addr_mem[15 ] = 8'd15 ;
assign addr_mem[16 ] = 8'd16 ;
assign addr_mem[17 ] = 8'd17 ;
assign addr_mem[18 ] = 8'd18 ;
assign addr_mem[19 ] = 8'd19 ;
assign addr_mem[20 ] = 8'd20 ;
assign addr_mem[21 ] = 8'd21 ;
assign addr_mem[22 ] = 8'd22 ;
assign addr_mem[23 ] = 8'd23 ;
assign addr_mem[24 ] = 8'd24 ;
assign addr_mem[25 ] = 8'd25 ;
assign addr_mem[26 ] = 8'd26 ;
assign addr_mem[27 ] = 8'd27 ;
assign addr_mem[28 ] = 8'd28 ;
assign addr_mem[29 ] = 8'd29 ;
assign addr_mem[30 ] = 8'd30 ;
assign addr_mem[31 ] = 8'd31 ;
assign addr_mem[32 ] = 8'd32 ;
assign addr_mem[33 ] = 8'd33 ;
assign addr_mem[34 ] = 8'd34 ;
assign addr_mem[35 ] = 8'd35 ;
assign addr_mem[36 ] = 8'd36 ;
assign addr_mem[37 ] = 8'd37 ;
assign addr_mem[38 ] = 8'd38 ;
assign addr_mem[39 ] = 8'd39 ;
assign addr_mem[40 ] = 8'd40 ;
assign addr_mem[41 ] = 8'd41 ;
assign addr_mem[42 ] = 8'd42 ;
assign addr_mem[43 ] = 8'd43 ;
assign addr_mem[44 ] = 8'd44 ;
assign addr_mem[45 ] = 8'd45 ;
assign addr_mem[46 ] = 8'd46 ;
assign addr_mem[47 ] = 8'd47 ;
assign addr_mem[48 ] = 8'd48 ;
assign addr_mem[49 ] = 8'd49 ;
assign addr_mem[50 ] = 8'd50 ;
assign addr_mem[51 ] = 8'd51 ;
assign addr_mem[52 ] = 8'd52 ;
assign addr_mem[53 ] = 8'd53 ;
assign addr_mem[54 ] = 8'd54 ;
assign addr_mem[55 ] = 8'd55 ;
assign addr_mem[56 ] = 8'd56 ;
assign addr_mem[57 ] = 8'd57 ;
assign addr_mem[58 ] = 8'd58 ;
assign addr_mem[59 ] = 8'd59 ;
assign addr_mem[60 ] = 8'd60 ;
assign addr_mem[61 ] = 8'd61 ;
assign addr_mem[62 ] = 8'd62 ;
assign addr_mem[63 ] = 8'd63 ;
assign addr_mem[64 ] = 8'd64 ;
assign addr_mem[65 ] = 8'd65 ;
assign addr_mem[66 ] = 8'd66 ;
assign addr_mem[67 ] = 8'd67 ;
assign addr_mem[68 ] = 8'd68 ;
assign addr_mem[69 ] = 8'd69 ;
assign addr_mem[70 ] = 8'd70 ;
assign addr_mem[71 ] = 8'd71 ;
assign addr_mem[72 ] = 8'd72 ;
assign addr_mem[73 ] = 8'd73 ;
assign addr_mem[74 ] = 8'd74 ;
assign addr_mem[75 ] = 8'd75 ;
assign addr_mem[76 ] = 8'd76 ;
assign addr_mem[77 ] = 8'd77 ;
assign addr_mem[78 ] = 8'd78 ;
assign addr_mem[79 ] = 8'd79 ;
assign addr_mem[80 ] = 8'd80 ;
assign addr_mem[81 ] = 8'd81 ;
assign addr_mem[82 ] = 8'd82 ;
assign addr_mem[83 ] = 8'd83 ;
assign addr_mem[84 ] = 8'd84 ;
assign addr_mem[85 ] = 8'd85 ;
assign addr_mem[86 ] = 8'd86 ;
assign addr_mem[87 ] = 8'd87 ;
assign addr_mem[88 ] = 8'd88 ;
assign addr_mem[89 ] = 8'd89 ;
assign addr_mem[90 ] = 8'd90 ;
assign addr_mem[91 ] = 8'd91 ;
assign addr_mem[92 ] = 8'd92 ;
assign addr_mem[93 ] = 8'd93 ;
assign addr_mem[94 ] = 8'd94 ;
assign addr_mem[95 ] = 8'd95 ;
assign addr_mem[96 ] = 8'd96 ;
assign addr_mem[97 ] = 8'd97 ;
assign addr_mem[98 ] = 8'd98 ;
assign addr_mem[99 ] = 8'd99 ;
assign addr_mem[100] = 8'd100;
assign addr_mem[101] = 8'd101;
assign addr_mem[102] = 8'd102;
assign addr_mem[103] = 8'd103;
assign addr_mem[104] = 8'd104;
assign addr_mem[105] = 8'd105;
assign addr_mem[106] = 8'd106;
assign addr_mem[107] = 8'd107;
assign addr_mem[108] = 8'd108;
assign addr_mem[109] = 8'd109;
assign addr_mem[110] = 8'd110;
assign addr_mem[111] = 8'd111;
assign addr_mem[112] = 8'd112;
assign addr_mem[113] = 8'd113;
assign addr_mem[114] = 8'd114;
assign addr_mem[115] = 8'd115;
assign addr_mem[116] = 8'd116;
assign addr_mem[117] = 8'd117;
assign addr_mem[118] = 8'd118;
assign addr_mem[119] = 8'd119;
assign addr_mem[120] = 8'd120;
assign addr_mem[121] = 8'd121;
assign addr_mem[122] = 8'd122;
assign addr_mem[123] = 8'd123;
assign addr_mem[124] = 8'd124;
assign addr_mem[125] = 8'd125;
assign addr_mem[126] = 8'd126;
assign addr_mem[127] = 8'd127;
assign addr_mem[128] = 8'd128;
assign addr_mem[129] = 8'd129;
assign addr_mem[130] = 8'd130;
assign addr_mem[131] = 8'd131;
assign addr_mem[132] = 8'd132;
assign addr_mem[133] = 8'd133;
assign addr_mem[134] = 8'd134;
assign addr_mem[135] = 8'd135;
assign addr_mem[136] = 8'd136;
assign addr_mem[137] = 8'd137;
assign addr_mem[138] = 8'd138;
assign addr_mem[139] = 8'd139;
assign addr_mem[140] = 8'd140;
assign addr_mem[141] = 8'd141;
assign addr_mem[142] = 8'd142;
assign addr_mem[143] = 8'd143;
assign addr_mem[144] = 8'd144;
assign addr_mem[145] = 8'd145;
assign addr_mem[146] = 8'd146;
assign addr_mem[147] = 8'd147;
assign addr_mem[148] = 8'd148;
assign addr_mem[149] = 8'd149;
assign addr_mem[150] = 8'd150;
assign addr_mem[151] = 8'd151;
assign addr_mem[152] = 8'd152;
assign addr_mem[153] = 8'd153;
assign addr_mem[154] = 8'd154;
assign addr_mem[155] = 8'd155;
assign addr_mem[156] = 8'd156;
assign addr_mem[157] = 8'd157;
assign addr_mem[158] = 8'd158;
assign addr_mem[159] = 8'd159;
assign addr_mem[160] = 8'd160;
assign addr_mem[161] = 8'd161;
assign addr_mem[162] = 8'd162;
assign addr_mem[163] = 8'd163;
assign addr_mem[164] = 8'd164;
assign addr_mem[165] = 8'd165;
assign addr_mem[166] = 8'd166;
assign addr_mem[167] = 8'd167;
assign addr_mem[168] = 8'd168;
assign addr_mem[169] = 8'd169;
assign addr_mem[170] = 8'd170;
assign addr_mem[171] = 8'd171;
assign addr_mem[172] = 8'd172;
assign addr_mem[173] = 8'd173;
assign addr_mem[174] = 8'd174;
assign addr_mem[175] = 8'd175;
assign addr_mem[176] = 8'd176;
assign addr_mem[177] = 8'd177;
assign addr_mem[178] = 8'd178;
assign addr_mem[179] = 8'd179;
assign addr_mem[180] = 8'd180;
assign addr_mem[181] = 8'd181;
assign addr_mem[182] = 8'd182;
assign addr_mem[183] = 8'd183;
assign addr_mem[184] = 8'd184;
assign addr_mem[185] = 8'd185;
assign addr_mem[186] = 8'd186;
assign addr_mem[187] = 8'd187;
assign addr_mem[188] = 8'd188;
assign addr_mem[189] = 8'd189;
assign addr_mem[190] = 8'd190;
assign addr_mem[191] = 8'd191;
assign addr_mem[192] = 8'd192;
assign addr_mem[193] = 8'd193;
assign addr_mem[194] = 8'd194;
assign addr_mem[195] = 8'd195;
assign addr_mem[196] = 8'd196;
assign addr_mem[197] = 8'd197;
assign addr_mem[198] = 8'd198;
assign addr_mem[199] = 8'd199;
assign addr_mem[200] = 8'd200;
assign addr_mem[201] = 8'd201;
assign addr_mem[202] = 8'd202;
assign addr_mem[203] = 8'd203;
assign addr_mem[204] = 8'd204;
assign addr_mem[205] = 8'd205;
assign addr_mem[206] = 8'd206;
assign addr_mem[207] = 8'd207;
assign addr_mem[208] = 8'd208;
assign addr_mem[209] = 8'd209;
assign addr_mem[210] = 8'd210;
assign addr_mem[211] = 8'd211;
assign addr_mem[212] = 8'd212;
assign addr_mem[213] = 8'd213;
assign addr_mem[214] = 8'd214;
assign addr_mem[215] = 8'd215;
assign addr_mem[216] = 8'd216;
assign addr_mem[217] = 8'd217;
assign addr_mem[218] = 8'd218;
assign addr_mem[219] = 8'd219;
assign addr_mem[220] = 8'd220;
assign addr_mem[221] = 8'd221;
assign addr_mem[222] = 8'd222;
assign addr_mem[223] = 8'd223;
assign addr_mem[224] = 8'd224;
assign addr_mem[225] = 8'd225;
assign addr_mem[226] = 8'd226;
assign addr_mem[227] = 8'd227;
assign addr_mem[228] = 8'd228;
assign addr_mem[229] = 8'd229;
assign addr_mem[230] = 8'd230;
assign addr_mem[231] = 8'd231;
assign addr_mem[232] = 8'd232;
assign addr_mem[233] = 8'd233;
assign addr_mem[234] = 8'd234;
assign addr_mem[235] = 8'd235;
assign addr_mem[236] = 8'd236;
assign addr_mem[237] = 8'd237;
assign addr_mem[238] = 8'd238;
assign addr_mem[239] = 8'd239;
assign addr_mem[240] = 8'd240;
assign addr_mem[241] = 8'd241;
assign addr_mem[242] = 8'd242;
assign addr_mem[243] = 8'd243;
assign addr_mem[244] = 8'd244;
assign addr_mem[245] = 8'd245;
assign addr_mem[246] = 8'd246;
assign addr_mem[247] = 8'd247;
assign addr_mem[248] = 8'd248;
assign addr_mem[249] = 8'd249;
assign addr_mem[250] = 8'd250;
assign addr_mem[251] = 8'd251;
assign addr_mem[252] = 8'd252;
assign addr_mem[253] = 8'd253;
assign addr_mem[254] = 8'd254;
assign addr_mem[255] = 8'd255;

always @(posedge I_CLK or negedge I_RESETN) 
begin
    if(!I_RESETN)
        mem_cnt <= 32'd0;
    else if(state == IDLE)
        mem_cnt <= 32'd0;
    else if(state == RD_DATA && wr_rd_stop == 2'd3)
    // else if(state == RD_RDTIP05 && rd_reg == 2'd3)
        mem_cnt <= mem_cnt + 1'b1;
    else
        mem_cnt <= mem_cnt;
end

//--------------------------------------------------------------------
always @(negedge I_RESETN or posedge I_CLK)
    if(~I_RESETN)
       start_dl <= 1'b0;     
    else
       start_dl <= start;        

////////////////////////////////////////////////////
assign error_flag=error_reg;
assign cstate_flag = cstate_flag_reg;
////////////////////////////////////////////////////

//--------------------------------------------------------------------
always @(posedge I_CLK or negedge I_RESETN) 
begin
    if(~I_RESETN)
        rdrpt_cnt <= 3'd0;
    else if(state == RD_CMD90_2)
        rdrpt_cnt <= 3'd0;
    else if(state == RD_DATA && wr_rd_stop == 2'd3)
        rdrpt_cnt <= rdrpt_cnt + 1'b1;
    else
        rdrpt_cnt <= rdrpt_cnt;
end

//--------------------------------------------------------------------
//状态机
//状态跳转
always @(posedge I_CLK or negedge I_RESETN) 
begin
    if(!I_RESETN)
        state <= IDLE;
    else 
        begin
            case(state)
                IDLE:
                    begin
                        if((start_dl == 1'b0) && (start == 1'b1))  //start rising edge
                            state <= PERSCALE0_REG;
                        else
                            state <= IDLE;
                    end
                PERSCALE0_REG:
                    begin
                        if(wr_reg == 2'd2)
                            state <= PERSCALE1_REG;
                        else
                            state <= PERSCALE0_REG;
                    end
                PERSCALE1_REG:
                    begin
                        if(wr_reg == 2'd2)
                            state <= MASTER_EN_REG;
                        else
                            state <= PERSCALE1_REG;
                    end
                MASTER_EN_REG:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_DEVICE_ADDRW;
                        else
                            state <= MASTER_EN_REG;
                    end
                
                
                //读操作开始    
                RD_DEVICE_ADDRW:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_CMD90_1;
                        else
                            state <= RD_DEVICE_ADDRW;
                    end    
                RD_CMD90_1:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_RDTIP01;
                        else
                            state <= RD_CMD90_1;
                    end    
                RD_RDTIP01:
                    begin
                        if(rd_reg == 2'd3)
                            state <= RD_REG_ADDR;
                        else
                            state <= RD_RDTIP01;
                    end    
                RD_REG_ADDR:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_CMD10_1;
                        else
                            state <= RD_REG_ADDR;
                    end    
                RD_CMD10_1:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_RDTIP02;
                        else
                            state <= RD_CMD10_1;
                    end    
                RD_RDTIP02:
                    begin
                        if(rd_reg == 2'd3)
                            state <= RD_DEVICE_ADDRR;
                        else
                            state <= RD_RDTIP02;
                    end    
                RD_DEVICE_ADDRR:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_CMD90_2;
                        else
                            state <= RD_DEVICE_ADDRR;
                    end    
                RD_CMD90_2:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_RDTIP03;
                        else
                            state <= RD_CMD90_2;
                    end    
                RD_RDTIP03:
                    begin
                        if(rd_reg == 2'd3)
                            state <= RD_CMD20;
                        else
                            state <= RD_RDTIP03;
                    end    
                RD_CMD20:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_RDTIP04;
                        else
                            state <= RD_CMD20;
                    end    
                RD_RDTIP04:
                    begin
                        if(rd_reg == 2'd3)
                            state <= RD_DATA;
                        else
                            state <= RD_RDTIP04;
                    end    
                RD_DATA: //读出数据
                    begin
                        if(mem_cnt>=NUM && wr_rd_stop == 2'd3)
                            state <= RD_CMD68;
                        else if(wr_rd_stop == 2'd3)
                            state <= RD_CMD20;
                        else
                            state <= RD_DATA;
                    end    
                RD_CMD68:
                    begin
                        if(wr_reg == 2'd2)
                            state <= RD_RDTIP05;
                        else
                            state <= RD_CMD68;
                    end    
                RD_RDTIP05:
                    begin
                        if(rd_reg == 2'd3)
                            begin
                                if(mem_cnt>=NUM)
                                    state <= IDLE;
                                else
                                    state <= RD_DEVICE_ADDRW;
                            end
                        else
                            state <= RD_RDTIP05;
                    end
                default:
                    begin
                        state <= IDLE;
                    end    
            endcase
        end
end

//状态输出
always @(negedge I_RESETN or posedge I_CLK) 
begin
    if(~I_RESETN)
        begin
            O_TX_EN <= 1'b0; 
            O_WADDR <= {3{1'b0}}; 
            O_WDATA <= {8{1'b0}}; 
            O_RX_EN <= 1'b0;
            O_RADDR <= {3{1'b0}};
            wr_reg <= 0;
            rd_reg <=0;
            sr_data0 <=0;
            wr_rd_stop <=0;
            
        end
    else 
        begin
            case(state)
                IDLE:
                    begin
                        O_TX_EN <= 1'b0; 
                        O_WADDR <= {3{1'b0}}; 
                        O_WDATA <= {8{1'b0}}; 
                        O_RX_EN <= 1'b0;
                        O_RADDR <= {3{1'b0}};
                        wr_reg <= 0;
                        rd_reg <=0;
                        wr_rd_stop <=0;
                    end
                PERSCALE0_REG:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= PRERLO_ADDR; 
                              O_WDATA <= CLK_Div_L; 
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end                
                        endcase    
                    end
                PERSCALE1_REG:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= PRERHI_ADDR; 
                              O_WDATA <= CLK_Div_H; 
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end
                MASTER_EN_REG:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CTR_ADDR; 
                              O_WDATA <= EN_IP; 
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end
                
                //读操作开始    
                RD_DEVICE_ADDRW:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= TXR_ADDR; 
                              O_WDATA <= {DEV_ADDR,WR};
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_CMD90_1:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CR_ADDR; 
                              O_WDATA <= STA_WR_CR;
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_RDTIP01:
                    begin
                        case(rd_reg)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= SR_ADDR;
                                rd_reg  <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                rd_reg  <= 2; 
                          end
                        2:
                          begin
                              if(~I_RDATA[1])
                                      rd_reg <= 3;
                                else
                                      rd_reg <= 0;                
                          end    
                        3:
                          begin
                                rd_reg <= 0;
                          end        
                        endcase
                    end    
                RD_REG_ADDR:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= TXR_ADDR; 
                              O_WDATA <= addr_mem[mem_cnt]; //memory address
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_CMD10_1:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CR_ADDR; 
                              O_WDATA <= WR_CR;
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_RDTIP02:
                    begin
                        case(rd_reg)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= SR_ADDR;
                                rd_reg  <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                rd_reg  <= 2; 
                          end
                        2:
                          begin
                              if(~I_RDATA[1])
                                      rd_reg <= 3;
                                else
                                      rd_reg <= 0;                
                          end    
                        3:
                          begin
                                rd_reg <= 0;
                          end        
                        endcase
                    end    
                RD_DEVICE_ADDRR:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= TXR_ADDR; 
                              O_WDATA <= {DEV_ADDR,RD};
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_CMD90_2:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CR_ADDR; 
                              O_WDATA <= STA_WR_CR;
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_RDTIP03:
                    begin
                        case(rd_reg)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= SR_ADDR;
                                rd_reg  <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                rd_reg  <= 2; 
                          end
                        2:
                          begin
                              if(~I_RDATA[1])
                                      rd_reg <= 3;
                                else
                                      rd_reg <= 0;                
                          end    
                        3:
                          begin
                                rd_reg <= 0;
                          end        
                        endcase
                    end    
                RD_CMD20:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CR_ADDR; 
                              O_WDATA <= RD_CR;
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_RDTIP04:
                    begin
                        case(rd_reg)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= SR_ADDR;
                                rd_reg  <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                rd_reg  <= 2; 
                          end
                        2:
                          begin
                              if(~I_RDATA[1])
                                      rd_reg <= 3;
                                else
                                      rd_reg <= 0;                
                          end    
                        3:
                          begin
                                rd_reg <= 0;
                          end        
                        endcase
                    end    
                RD_DATA: //读出数据
                    begin
                        case(wr_rd_stop)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= RXR_ADDR;
                                wr_rd_stop <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                wr_rd_stop <= 2; 
                          end
                        2:
                          begin
                                sr_data0 <= I_RDATA;
                                wr_rd_stop <= 3;
                          end    
                        3:
                          begin
                                wr_rd_stop <= 0;
                                if(sr_data0 == TX_DATA)
                                  error_reg <= 0;
                              else
                                  error_reg <= 1;
                          end        
                        endcase
                    end    
                RD_CMD68:
                    begin
                        case(wr_reg)
                        0:
                          begin
                              O_TX_EN <= 1'b1; 
                              O_WADDR <= CR_ADDR; 
                              O_WDATA <= STP_RD_NCR;
                                wr_reg <= 1;
                          end
                        1:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 2; 
                          end
                        2:
                          begin
                                wr_reg <= 0;
                          end
                        default:
                          begin
                                O_TX_EN <= 1'b0;
                                O_WADDR <= 1'b0;
                                O_WDATA <= 0;
                                wr_reg <= 0;
                          end            
                        endcase
                    end    
                RD_RDTIP05:
                    begin
                        case(rd_reg)
                        0:
                          begin
                              O_RX_EN <= 1'b1;
                              O_RADDR <= SR_ADDR;
                                rd_reg  <= 1;
                          end
                        1:
                          begin
                              O_RX_EN <= 1'b0;
                                rd_reg  <= 2; 
                          end
                        2:
                          begin
                              if(~I_RDATA[1])
                                      rd_reg <= 3;
                                else
                                      rd_reg <= 0;                
                          end    
                        3:
                          begin
                                rd_reg <= 0;
                          end        
                        endcase
                    end
                default:
                    begin
                        O_TX_EN <= 1'b0; 
                        O_WADDR <= {3{1'b0}}; 
                        O_WDATA <= {8{1'b0}}; 
                        O_RX_EN <= 1'b0;
                        O_RADDR <= {3{1'b0}};
                        wr_reg <= 0;
                        rd_reg <=0;
                        wr_rd_stop <=0;
                    end    
            endcase
        end
end





endmodule                    
