//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date        | AUTHOR     |    DESCRIPTION
// ---------------------------------------------------------------------
//  1.0    |  10-Oct-2020 |            |     initial 
// ---------------------------------------------------------------------

`timescale 1ns/1ps 

module tb();
//========================================================
//parameters
parameter CLK_FREQ   = 50.000;//ddr reference clock frequency, unit: MHz
parameter CLK_PERIOD = 1000.0/CLK_FREQ; //unit: ns 

//=======================================================
reg  clk;      // 50M
reg  rst_n      ;

//apb reg
wire [31:0] paddr;
wire        penable;
wire [31:0] prdata;
wire        pready;
wire        psel;
wire [31:0] pwdata;
wire        pwrite;

wire        ma_we;
wire        cmd_finish;

//----------------
wire    scl;
wire    sda;

//========================================================
pullup p1(scl); // pullup scl line
pullup p2(sda); // pullup sda line

GSR GSR(.GSRI(1'b1));

initial 
begin
    $fsdbDumpfile("tb.fsdb");
    $fsdbDumpvars;
end

//==============================================  
//rst_n     
initial
  begin
    rst_n=1'b0;
    
    #2000;
    rst_n=1'b1;	

    #30000000
	$finish;
end

//----------------------------------------------------
//ref clk
initial 
begin
	clk = 1'b0;
end

always  #(CLK_PERIOD/2.0) clk = ~clk;

//==================================================
//Generate command 
//Modify instruction in this module
cmd_gen cmd_gen_inst
(
    .clk               (clk       ),
    .rst_n             (rst_n     ),
    .cmd_finish        (cmd_finish),
    .paddr             (paddr     ),//apb port 
    .penable           (penable   ),
    .prdata            (prdata    ),
    .pready            (1'b1      ),
    .psel              (psel      ),
    .pwdata            (pwdata    ),
    .pwrite            (pwrite    )
);

//===========================================================================
EDID_PROM_Top  EDID_PROM_Top_inst 
( 
	.I_clk      (clk        ),//>= 5MHz, <=200MHz 
	.I_rst_n    (rst_n      ), 
    .I_mema_clk (clk        ),
    .I_mema_we  (ma_we      ),
    .I_mema_addr(paddr[7:0] ),
    .I_mema_di  (pwdata[7:0]),
	.I_scl      (scl        ),
	.IO_sda     (sda        )
);

assign ma_we = pwrite & psel & penable;
assign prdata = 32'd0;

//==================================================
//I2C read
gw_i2c_master gw_i2c_master_inst  
(
    .clk  (clk  ),
    .rst_n(rst_n),
    .start(cmd_finish),
    .scl  (scl  ),
    .sda  (sda  )
);

endmodule
